﻿using MSTSCLib;
using System;
using System.Drawing;
using System.Windows.Forms;

namespace RDP7Plugin
{
    public partial class FormRemotePC : Form
    {
        private static readonly int p_Width = Screen.PrimaryScreen.WorkingArea.Width;
        private static readonly int P_Height = Screen.PrimaryScreen.WorkingArea.Height;

        public FormRemotePC()
        {
            InitializeComponent();

            Height = P_Height;
            Width = p_Width;
            Location = new Point(1, 1);
            Icon = Icon.FromHandle(Properties.Resources.network.GetHicon());
        }

        public FormRemotePC(string Server, string UserName, string Password, string Comment=null) : this()
        {
            Text += $" {Server}";
            if (!string.IsNullOrWhiteSpace(Comment))
                Text += $" ({Comment})";

            if (!string.IsNullOrWhiteSpace(Server))
            {
                string ServerIPorName = Server;
                int ServerPort = 3389;

                string[] split = Server.Split(':');
                if (split.Length == 2)
                {
                    ServerIPorName = split[0];
                    int.TryParse(split[1], out ServerPort);
                }

                axMsTscAxNotSafeForScripting.BeginInit();

                axMsTscAxNotSafeForScripting.OnConnected += AxMsTscAxNotSafeForScripting_OnConnected;
                axMsTscAxNotSafeForScripting.OnWarning += AxMsTscAxNotSafeForScripting_OnWarning;
                axMsTscAxNotSafeForScripting.OnFatalError += AxMsTscAxNotSafeForScripting_OnFatalError;
                axMsTscAxNotSafeForScripting.OnLogonError += AxMsTscAxNotSafeForScripting_OnLogonError;

                axMsTscAxNotSafeForScripting.DesktopHeight = ClientSize.Height;
                axMsTscAxNotSafeForScripting.DesktopWidth = ClientSize.Width;
                axMsTscAxNotSafeForScripting.Server = ServerIPorName;
                axMsTscAxNotSafeForScripting.UserName = UserName;

                if (axMsTscAxNotSafeForScripting.AdvancedSettings is IMsRdpClientAdvancedSettings2 settings2)
                {
                    settings2.ClearTextPassword = Password;

                    settings2.RDPPort = ServerPort;

                    settings2.SmartSizing = true;
                    settings2.RedirectDrives = true;
                    settings2.RedirectPrinters = false;
                    settings2.RedirectPorts = false;
                    settings2.RedirectSmartCards = false;
                    settings2.EnableAutoReconnect = true;
                    settings2.EncryptionEnabled = 1;
                    //settings2.Compress = 1;

                    if (axMsTscAxNotSafeForScripting.AdvancedSettings is IMsRdpClientAdvancedSettings5 settings5)
                    {
                        settings5.RedirectClipboard = true;
                    }

                    if (axMsTscAxNotSafeForScripting.AdvancedSettings is IMsRdpClientAdvancedSettings7 settings7)
                    {
                        settings7.EnableCredSspSupport = true;
                        settings7.NegotiateSecurityLayer = true;
                    }
                }
                axMsTscAxNotSafeForScripting.Connect();
                axMsTscAxNotSafeForScripting.EndInit();
                return;
            }
            labelStatus.Text = "Can't connect to server :(";
        }

        private void AxMsTscAxNotSafeForScripting_OnConnected(object sender, EventArgs e)
        {
            if (axMsTscAxNotSafeForScripting.Connected > 0)
            {
                axMsTscAxNotSafeForScripting.Visible = true;
                labelStatus.Visible = false;
            }

        }

        private void AxMsTscAxNotSafeForScripting_OnLogonError(object sender, AxMSTSCLib.IMsTscAxEvents_OnLogonErrorEvent e)
        {
            labelStatus.Text = $"Logon error: {e.lError}";
        }

        private void AxMsTscAxNotSafeForScripting_OnFatalError(object sender, AxMSTSCLib.IMsTscAxEvents_OnFatalErrorEvent e)
        {
            labelStatus.Text = $"Fatal: {e.errorCode}";
        }

        private void AxMsTscAxNotSafeForScripting_OnWarning(object sender, AxMSTSCLib.IMsTscAxEvents_OnWarningEvent e)
        {
            labelStatus.Text = $"Warning: {e.warningCode}";
        }

        private void FormRemotePC_FormClosing(object sender, FormClosingEventArgs e)
        {
            axMsTscAxNotSafeForScripting.OnConnected -= AxMsTscAxNotSafeForScripting_OnConnected;
            axMsTscAxNotSafeForScripting.OnWarning -= AxMsTscAxNotSafeForScripting_OnWarning;
            axMsTscAxNotSafeForScripting.OnFatalError -= AxMsTscAxNotSafeForScripting_OnFatalError;
            axMsTscAxNotSafeForScripting.OnLogonError -= AxMsTscAxNotSafeForScripting_OnLogonError;
            if (axMsTscAxNotSafeForScripting.Connected > 0)
            {
                axMsTscAxNotSafeForScripting.Disconnect();
                labelStatus.Text = "Disconnecting...";
                labelStatus.Visible = true;
                axMsTscAxNotSafeForScripting.Visible = false;
            }
            axMsTscAxNotSafeForScripting.Dispose();
        }
    }
}
