﻿using KeePass.Plugins;
using KeePassLib;
using System;
using System.Windows.Forms;

namespace RDP7Plugin
{
    public sealed class RDP7PluginExt : Plugin
    {
        private const string URLS_NAME = "m_ctxEntryUrl";
        private ToolStripSeparator m_UrlsRdpConnectSeparator = null;
        private ToolStripMenuItem m_UrlsRdpConnectMenuItem = null;
        private IPluginHost m_host = null;

        public override bool Initialize(IPluginHost host)
        {
            if (host != null)
            {
                m_host = host;

                if (m_host.MainWindow.EntryContextMenu.Items[URLS_NAME] is ToolStripMenuItem urlsMenu)
                {
                    if (urlsMenu.IsOnDropDown)
                    {
                        urlsMenu.DropDownOpening += UrlsMenu_DropDownOpening;

                        // Add a separator at the bottom
                        m_UrlsRdpConnectSeparator = new ToolStripSeparator();
                        urlsMenu.DropDown.Items.Add(m_UrlsRdpConnectSeparator);
                        // Add menu item 
                        m_UrlsRdpConnectMenuItem = new ToolStripMenuItem()
                        {
                            Text = "RDP 7 Connect...",
                            Image = Properties.Resources.network
                        };

                        m_UrlsRdpConnectMenuItem.Click += M_UrlsRdpConnectMenuItem_Click;

                        urlsMenu.DropDown.Items.Add(m_UrlsRdpConnectMenuItem);

                        return true;
                    }
                }
            }
            return false;
        }

        public override void Terminate()
        {
            // Remove all of our menu items
            if (m_host.MainWindow.EntryContextMenu.Items[URLS_NAME] is ToolStripMenuItem urlsMenu)
            {
                m_UrlsRdpConnectMenuItem.Click -= M_UrlsRdpConnectMenuItem_Click;
                urlsMenu.DropDown.Items.Remove(m_UrlsRdpConnectMenuItem);
                urlsMenu.DropDown.Items.Remove(m_UrlsRdpConnectSeparator);
            }
        }

        private void UrlsMenu_DropDownOpening(object sender, EventArgs e)
        {
            if (m_UrlsRdpConnectMenuItem != null)
            {
                if (m_host.MainWindow.GetSelectedEntriesCount() == 1)
                    m_UrlsRdpConnectMenuItem.Enabled = true;
                else
                    m_UrlsRdpConnectMenuItem.Enabled = false;
            }
        }

        private void M_UrlsRdpConnectMenuItem_Click(object sender, EventArgs e)
        {
            KeePassLib.PwEntry entry = m_host.MainWindow.GetSelectedEntry(false);
            if (entry != null)
            {
                TryGetStringFromEntry(entry, "Title", out string Title); if (Title.Length > 20) Title = $" {Title.Substring(0, 17)}...";
                TryGetStringFromEntry(entry, "UserName", out string UserName);
                TryGetStringFromEntry(entry, "Password", out string Password);
                TryGetStringFromEntry(entry, "URL", out string URL);

                FormRemotePC f = new FormRemotePC(URL, UserName, Password, Title);
                f.Show();
            }
            else
                MessageBox.Show("Can't get selected entry", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void TryGetStringFromEntry(PwEntry entry, string entryName, out string data)
        {
            if (entry.Strings.Exists(entryName))
                data = entry.Strings.Get(entryName).ReadString();
            else
                data = string.Empty;
        }
    }
}
